@@ -21,13 +21,12 @@ Set up the cloud configuration at ``/etc/salt/cloud.providers`` or
 '''
 
 # Import python libs
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 import time
 import pprint
 import logging
 
 # Import Salt libs
-import salt.utils
 from salt._compat import ElementTree as ET
 # pylint: disable=import-error,no-name-in-module
 from salt.ext.six.moves.urllib.error import URLError
@@ -51,6 +50,9 @@ from salt.exceptions import (
     SaltCloudExecutionTimeout
 )
 
+# Import 3rd-party libs
+from salt.ext import six
+
 # Get logging started
 log = logging.getLogger(__name__)
 
@@ -174,7 +176,7 @@ def get_image(vm_):
         'image', vm_, __opts__, search_global=False
     )
     for image in images:
-        if str(vm_image) in (images[image]['name'], images[image]['id']):
+        if six.text_type(vm_image) in (images[image]['name'], images[image]['id']):
             return images[image]['id']
     raise SaltCloudNotFound('The specified image could not be found.')
 
@@ -291,17 +293,16 @@ def create(vm_):
         transport=__opts__['transport']
     )
 
-    log.info('Creating Cloud VM {0}'.format(vm_['name']))
+    log.info('Creating Cloud VM %s', vm_['name'])
 
     try:
         data = create_node(vm_)
     except Exception as exc:
         log.error(
-            'Error creating {0} on PARALLELS\n\n'
+            'Error creating %s on PARALLELS\n\n'
             'The following exception was thrown when trying to '
-            'run the initial deployment: \n{1}'.format(
-                vm_['name'], str(exc)
-            ),
+            'run the initial deployment: \n%s',
+            vm_['name'], exc,
             # Show the traceback if the debug logging level is enabled
             exc_info_on_loglevel=logging.DEBUG
         )
@@ -338,7 +339,7 @@ def create(vm_):
         except SaltCloudSystemExit:
             pass
         finally:
-            raise SaltCloudSystemExit(str(exc))
+            raise SaltCloudSystemExit(six.text_type(exc))
 
     comps = data['network']['public-ip']['address'].split('/')
     public_ip = comps[0]
@@ -346,11 +347,10 @@ def create(vm_):
     vm_['ssh_host'] = public_ip
     ret = __utils__['cloud.bootstrap'](vm_, __opts__)
 
-    log.info('Created Cloud VM \'{0[name]}\''.format(vm_))
+    log.info('Created Cloud VM \'%s\'', vm_['name'])
     log.debug(
-        '\'{0[name]}\' VM creation details:\n{1}'.format(
-            vm_, pprint.pformat(data)
-        )
+        '\'%s\' VM creation details:\n%s',
+        vm_['name'], pprint.pformat(data)
     )
 
     __utils__['cloud.fire_event'](
@@ -397,7 +397,7 @@ def query(action=None, command=None, args=None, method='GET', data=None):
         args = {}
 
     kwargs = {'data': data}
-    if isinstance(data, str) and '<?xml' in data:
+    if isinstance(data, six.string_types) and '<?xml' in data:
         kwargs['headers'] = {
             'Content-type': 'application/xml',
         }
@@ -410,17 +410,13 @@ def query(action=None, command=None, args=None, method='GET', data=None):
 
     req.get_method = lambda: method
 
-    log.debug('{0} {1}'.format(method, req.get_full_url()))
+    log.debug('%s %s', method, req.get_full_url())
     if data:
         log.debug(data)
 
     try:
         result = _urlopen(req)
-        log.debug(
-            'PARALLELS Response Status Code: {0}'.format(
-                result.getcode()
-            )
-        )
+        log.debug('PARALLELS Response Status Code: %s', result.getcode())
 
         if 'content-length' in result.headers:
             content = result.read()
@@ -430,12 +426,7 @@ def query(action=None, command=None, args=None, method='GET', data=None):
 
         return {}
     except URLError as exc:
-        log.error(
-            'PARALLELS Response Status Code: {0} {1}'.format(
-                exc.code,
-                exc.msg
-            )
-        )
+        log.error('PARALLELS Response Status Code: %s %s', exc.code, exc.msg)
         root = ET.fromstring(exc.read())
         log.error(root)
         return {'error': root}
