@@ -407,10 +407,10 @@ class FileserverUpdate(salt.utils.process.SignalHandlingMultiprocessingProcess):
                     interval = self.opts[interval_key]
                 except KeyError:
                     interval = DEFAULT_INTERVAL
-                    log.error(
-                        '%s key missing from master configuration. This is '
-                        'a bug, please report it. Falling back to default '
-                        'interval of %d seconds', interval_key, interval
+                    log.warning(
+                        '%s key missing from configuration. Falling back to '
+                        'default interval of %d seconds',
+                        interval_key, interval
                     )
                 self.buckets.setdefault(
                     interval, OrderedDict())[(backend, update_func)] = None
@@ -578,9 +578,8 @@ class Master(SMaster):
         # Check to see if we need to create a pillar cache dir
         if self.opts['pillar_cache'] and not os.path.isdir(os.path.join(self.opts['cachedir'], 'pillar_cache')):
             try:
-                prev_umask = os.umask(0o077)
-                os.mkdir(os.path.join(self.opts['cachedir'], 'pillar_cache'))
-                os.umask(prev_umask)
+                with salt.utils.files.set_umask(0o077):
+                    os.mkdir(os.path.join(self.opts['cachedir'], 'pillar_cache'))
             except OSError:
                 pass
 
@@ -601,7 +600,8 @@ class Master(SMaster):
                                 new_opts,
                                 repo['git'],
                                 per_remote_overrides=salt.pillar.git_pillar.PER_REMOTE_OVERRIDES,
-                                per_remote_only=salt.pillar.git_pillar.PER_REMOTE_ONLY)
+                                per_remote_only=salt.pillar.git_pillar.PER_REMOTE_ONLY,
+                                global_only=salt.pillar.git_pillar.GLOBAL_ONLY)
                         except salt.exceptions.FileserverConfigError as exc:
                             critical_errors.append(exc.strerror)
                 finally:
@@ -1451,7 +1451,7 @@ class AESFuncs(object):
             if load['loc']:
                 fp_.seek(load['loc'])
 
-            fp_.write(load['data'])
+            fp_.write(salt.utils.stringutils.to_bytes(load['data']))
         return True
 
     def _pillar(self, load):
