@@ -4,18 +4,23 @@ Salt module to manage Unix mounts and the fstab file
 '''
 
 # Import python libs
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 import os
 import re
 import logging
 
 # Import salt libs
-import salt.utils
-from salt.utils import which as _which
+import salt.utils.args
+import salt.utils.data
+import salt.utils.files
+import salt.utils.path
+import salt.utils.platform
+import salt.utils.mount
+import salt.utils.stringutils
 from salt.exceptions import CommandNotFoundError, CommandExecutionError
 
 # Import 3rd-party libs
-import salt.ext.six as six
+from salt.ext import six
 from salt.ext.six.moves import filter, zip  # pylint: disable=import-error,redefined-builtin
 
 # Set up logger
@@ -30,7 +35,7 @@ def __virtual__():
     Only load on POSIX-like systems
     '''
     # Disable on Windows, a specific file module exists:
-    if salt.utils.is_windows():
+    if salt.utils.platform.is_windows():
         return (False, 'The mount module cannot be loaded: not a POSIX-like system.')
     return True
 
@@ -58,9 +63,9 @@ def _active_mountinfo(ret):
 
     blkid_info = __salt__['disk.blkid']()
 
-    with salt.utils.fopen(filename) as ifile:
+    with salt.utils.files.fopen(filename) as ifile:
         for line in ifile:
-            comps = line.split()
+            comps = salt.utils.stringutils.to_unicode(line).split()
             device = comps[2].split(':')
             # each line can have any number of
             # optional parameters, we use the
@@ -100,9 +105,9 @@ def _active_mounts(ret):
         msg = 'File not readable {0}'
         raise CommandExecutionError(msg.format(filename))
 
-    with salt.utils.fopen(filename) as ifile:
+    with salt.utils.files.fopen(filename) as ifile:
         for line in ifile:
-            comps = line.split()
+            comps = salt.utils.stringutils.to_unicode(line).split()
             ret[comps[1]] = {'device': comps[0],
                              'alt_device': _list.get(comps[1], None),
                              'fstype': comps[2],
@@ -168,8 +173,8 @@ def _active_mounts_openbsd(ret):
             ret[comps[3]] = {'device': comps[0],
                          'fstype': comps[5],
                          'opts': _resolve_user_group_names(parens[1].split(", ")),
-                         'major': str(nod[4].strip(",")),
-                         'minor': str(nod[5]),
+                         'major': six.text_type(nod[4].strip(",")),
+                         'minor': six.text_type(nod[5]),
                          'device_uuid': parens[0]}
         else:
             ret[comps[2]] = {'device': comps[0],
@@ -206,7 +211,7 @@ def _resolve_user_group_names(opts):
                 _info = __salt__[name_id_opts[_param]](_givenid)
                 if _info and _param in _info:
                     _id = _info[_param]
-            opts[ind] = _param + '=' + str(_id)
+            opts[ind] = _param + '=' + six.text_type(_id)
     return opts
 
 
@@ -252,7 +257,9 @@ class _fstab_entry(object):
     '''
 
     class ParseError(ValueError):
-        '''Error raised when a line isn't parsible as an fstab entry'''
+        '''
+        Error raised when a line isn't parsible as an fstab entry
+        '''
 
     fstab_keys = ('device', 'name', 'fstype', 'opts', 'dump', 'pass_num')
 
@@ -283,31 +290,43 @@ class _fstab_entry(object):
         return cls.fstab_format.format(**entry)
 
     def __str__(self):
-        '''string value, only works for full repr'''
+        '''
+        String value, only works for full repr
+        '''
         return self.dict_to_line(self.criteria)
 
     def __repr__(self):
-        '''always works'''
-        return str(self.criteria)
+        '''
+        Always works
+        '''
+        return repr(self.criteria)
 
     def pick(self, keys):
-        '''returns an instance with just those keys'''
+        '''
+        Returns an instance with just those keys
+        '''
         subset = dict([(key, self.criteria[key]) for key in keys])
         return self.__class__(**subset)
 
     def __init__(self, **criteria):
-        '''Store non-empty, non-null values to use as filter'''
+        '''
+        Store non-empty, non-null values to use as filter
+        '''
         items = [key_value for key_value in six.iteritems(criteria) if key_value[1] is not None]
-        items = [(key_value1[0], str(key_value1[1])) for key_value1 in items]
+        items = [(key_value1[0], six.text_type(key_value1[1])) for key_value1 in items]
         self.criteria = dict(items)
 
     @staticmethod
     def norm_path(path):
-        '''Resolve equivalent paths equivalently'''
+        '''
+        Resolve equivalent paths equivalently
+        '''
         return os.path.normcase(os.path.normpath(path))
 
     def match(self, line):
-        '''compare potentially partial criteria against line'''
+        '''
+        Compare potentially partial criteria against line
+        '''
         entry = self.dict_from_line(line)
         for key, value in six.iteritems(self.criteria):
             if entry[key] != value:
@@ -334,7 +353,7 @@ class _vfstab_entry(object):
         '''Error raised when a line isn't parsible as an fstab entry'''
 
     vfstab_keys = ('device', 'device_fsck', 'name', 'fstype', 'pass_fsck', 'mount_at_boot', 'opts')
-    ## NOTE: weird formatting to match default spacing on Solaris
+    # NOTE: weird formatting to match default spacing on Solaris
     vfstab_format = '{device:<11} {device_fsck:<3} {name:<19} {fstype:<8} {pass_fsck:<3} {mount_at_boot:<6} {opts}\n'
 
     @classmethod
@@ -357,31 +376,43 @@ class _vfstab_entry(object):
         return cls.vfstab_format.format(**entry)
 
     def __str__(self):
-        '''string value, only works for full repr'''
+        '''
+        String value, only works for full repr
+        '''
         return self.dict_to_line(self.criteria)
 
     def __repr__(self):
-        '''always works'''
-        return str(self.criteria)
+        '''
+        Always works
+        '''
+        return repr(self.criteria)
 
     def pick(self, keys):
-        '''returns an instance with just those keys'''
+        '''
+        Returns an instance with just those keys
+        '''
         subset = dict([(key, self.criteria[key]) for key in keys])
         return self.__class__(**subset)
 
     def __init__(self, **criteria):
-        '''Store non-empty, non-null values to use as filter'''
+        '''
+        Store non-empty, non-null values to use as filter
+        '''
         items = [key_value for key_value in six.iteritems(criteria) if key_value[1] is not None]
-        items = [(key_value1[0], str(key_value1[1])) for key_value1 in items]
+        items = [(key_value1[0], six.text_type(key_value1[1])) for key_value1 in items]
         self.criteria = dict(items)
 
     @staticmethod
     def norm_path(path):
-        '''Resolve equivalent paths equivalently'''
+        '''
+        Resolve equivalent paths equivalently
+        '''
         return os.path.normcase(os.path.normpath(path))
 
     def match(self, line):
-        '''compare potentially partial criteria against line'''
+        '''
+        Compare potentially partial criteria against line
+        '''
         entry = self.dict_from_line(line)
         for key, value in six.iteritems(self.criteria):
             if entry[key] != value:
@@ -403,11 +434,12 @@ def fstab(config='/etc/fstab'):
     ret = {}
     if not os.path.isfile(config):
         return ret
-    with salt.utils.fopen(config) as ifile:
+    with salt.utils.files.fopen(config) as ifile:
         for line in ifile:
+            line = salt.utils.stringutils.to_unicode(line)
             try:
                 if __grains__['kernel'] == 'SunOS':
-                    ## Note: comments use in default vfstab file!
+                    # Note: comments use in default vfstab file!
                     if line[0] == '#':
                         continue
                     entry = _vfstab_entry.dict_from_line(
@@ -441,7 +473,7 @@ def vfstab(config='/etc/vfstab'):
 
         salt '*' mount.vfstab
     '''
-    ## NOTE: vfstab is a wrapper for fstab
+    # NOTE: vfstab is a wrapper for fstab
     return fstab(config)
 
 
@@ -465,8 +497,9 @@ def rm_fstab(name, device, config='/etc/fstab'):
 
     lines = []
     try:
-        with salt.utils.fopen(config, 'r') as ifile:
+        with salt.utils.files.fopen(config, 'r') as ifile:
             for line in ifile:
+                line = salt.utils.stringutils.to_unicode(line)
                 try:
                     if criteria.match(line):
                         modified = True
@@ -480,15 +513,15 @@ def rm_fstab(name, device, config='/etc/fstab'):
 
     except (IOError, OSError) as exc:
         msg = "Couldn't read from {0}: {1}"
-        raise CommandExecutionError(msg.format(config, str(exc)))
+        raise CommandExecutionError(msg.format(config, exc))
 
     if modified:
         try:
-            with salt.utils.fopen(config, 'w+') as ofile:
-                ofile.writelines(lines)
+            with salt.utils.files.fopen(config, 'wb') as ofile:
+                ofile.writelines(salt.utils.data.encode(lines))
         except (IOError, OSError) as exc:
             msg = "Couldn't write to {0}: {1}"
-            raise CommandExecutionError(msg.format(config, str(exc)))
+            raise CommandExecutionError(msg.format(config, exc))
 
     # Note: not clear why we always return 'True'
     # --just copying previous behavior at this point...
@@ -594,8 +627,9 @@ def set_fstab(
         raise CommandExecutionError('Bad config file "{0}"'.format(config))
 
     try:
-        with salt.utils.fopen(config, 'r') as ifile:
+        with salt.utils.files.fopen(config, 'r') as ifile:
             for line in ifile:
+                line = salt.utils.stringutils.to_unicode(line)
                 try:
                     if criteria.match(line):
                         # Note: If ret isn't None here,
@@ -605,7 +639,7 @@ def set_fstab(
                             lines.append(line)
                         else:
                             ret = 'change'
-                            lines.append(str(entry))
+                            lines.append(six.text_type(entry))
                     else:
                         lines.append(line)
 
@@ -614,19 +648,19 @@ def set_fstab(
 
     except (IOError, OSError) as exc:
         msg = 'Couldn\'t read from {0}: {1}'
-        raise CommandExecutionError(msg.format(config, str(exc)))
+        raise CommandExecutionError(msg.format(config, exc))
 
     # add line if not present or changed
     if ret is None:
-        lines.append(str(entry))
+        lines.append(six.text_type(entry))
         ret = 'new'
 
     if ret != 'present':  # ret in ['new', 'change']:
-        if not salt.utils.test_mode(test=test, **kwargs):
+        if not salt.utils.args.test_mode(test=test, **kwargs):
             try:
-                with salt.utils.fopen(config, 'w+') as ofile:
+                with salt.utils.files.fopen(config, 'wb') as ofile:
                     # The line was changed, commit it!
-                    ofile.writelines(lines)
+                    ofile.writelines(salt.utils.data.encode(lines))
             except (IOError, OSError):
                 msg = 'File not writable {0}'
                 raise CommandExecutionError(msg.format(config))
@@ -722,8 +756,9 @@ def set_vfstab(
         raise CommandExecutionError('Bad config file "{0}"'.format(config))
 
     try:
-        with salt.utils.fopen(config, 'r') as ifile:
+        with salt.utils.files.fopen(config, 'r') as ifile:
             for line in ifile:
+                line = salt.utils.stringutils.to_unicode(line)
                 try:
                     if criteria.match(line):
                         # Note: If ret isn't None here,
@@ -733,7 +768,7 @@ def set_vfstab(
                             lines.append(line)
                         else:
                             ret = 'change'
-                            lines.append(str(entry))
+                            lines.append(six.text_type(entry))
                     else:
                         lines.append(line)
 
@@ -742,19 +777,19 @@ def set_vfstab(
 
     except (IOError, OSError) as exc:
         msg = 'Couldn\'t read from {0}: {1}'
-        raise CommandExecutionError(msg.format(config, str(exc)))
+        raise CommandExecutionError(msg.format(config, exc))
 
     # add line if not present or changed
     if ret is None:
-        lines.append(str(entry))
+        lines.append(six.text_type(entry))
         ret = 'new'
 
     if ret != 'present':  # ret in ['new', 'change']:
-        if not salt.utils.test_mode(test=test, **kwargs):
+        if not salt.utils.args.test_mode(test=test, **kwargs):
             try:
-                with salt.utils.fopen(config, 'w+') as ofile:
+                with salt.utils.files.fopen(config, 'wb') as ofile:
                     # The line was changed, commit it!
-                    ofile.writelines(lines)
+                    ofile.writelines(salt.utils.data.encode(lines))
             except (IOError, OSError):
                 msg = 'File not writable {0}'
                 raise CommandExecutionError(msg.format(config))
@@ -778,8 +813,9 @@ def rm_automaster(name, device, config='/etc/auto_salt'):
     # The entry is present, get rid of it
     lines = []
     try:
-        with salt.utils.fopen(config, 'r') as ifile:
+        with salt.utils.files.fopen(config, 'r') as ifile:
             for line in ifile:
+                line = salt.utils.stringutils.to_unicode(line)
                 if line.startswith('#'):
                     # Commented
                     lines.append(line)
@@ -808,14 +844,14 @@ def rm_automaster(name, device, config='/etc/auto_salt'):
                 lines.append(line)
     except (IOError, OSError) as exc:
         msg = "Couldn't read from {0}: {1}"
-        raise CommandExecutionError(msg.format(config, str(exc)))
+        raise CommandExecutionError(msg.format(config, exc))
 
     try:
-        with salt.utils.fopen(config, 'w+') as ofile:
-            ofile.writelines(lines)
+        with salt.utils.files.fopen(config, 'wb') as ofile:
+            ofile.writelines(salt.utils.data.encode(lines))
     except (IOError, OSError) as exc:
         msg = "Couldn't write to {0}: {1}"
-        raise CommandExecutionError(msg.format(config, str(exc)))
+        raise CommandExecutionError(msg.format(config, exc))
 
     # Update automount
     __salt__['cmd.run']('automount -cv')
@@ -860,8 +896,9 @@ def set_automaster(
         device_fmt = device_fmt.replace(fstype, "")
 
     try:
-        with salt.utils.fopen(config, 'r') as ifile:
+        with salt.utils.files.fopen(config, 'r') as ifile:
             for line in ifile:
+                line = salt.utils.stringutils.to_unicode(line)
                 if line.startswith('#'):
                     # Commented
                     lines.append(line)
@@ -890,26 +927,26 @@ def set_automaster(
                         comps[2] = device_fmt
                     if change:
                         log.debug(
-                            'auto_master entry for mount point {0} needs to be '
-                            'updated'.format(name)
-                       )
+                            'auto_master entry for mount point %s needs to be '
+                            'updated', name
+                        )
                         newline = (
                             '{0}\t{1}\t{2}\n'.format(
                                 name, type_opts, device_fmt)
-                       )
+                        )
                         lines.append(newline)
                 else:
                     lines.append(line)
     except (IOError, OSError) as exc:
         msg = 'Couldn\'t read from {0}: {1}'
-        raise CommandExecutionError(msg.format(config, str(exc)))
+        raise CommandExecutionError(msg.format(config, exc))
 
     if change:
-        if not salt.utils.test_mode(test=test, **kwargs):
+        if not salt.utils.args.test_mode(test=test, **kwargs):
             try:
-                with salt.utils.fopen(config, 'w+') as ofile:
+                with salt.utils.files.fopen(config, 'wb') as ofile:
                     # The line was changed, commit it!
-                    ofile.writelines(lines)
+                    ofile.writelines(salt.utils.data.encode(lines))
             except (IOError, OSError):
                 msg = 'File not writable {0}'
                 raise CommandExecutionError(msg.format(config))
@@ -921,7 +958,7 @@ def set_automaster(
             # The right entry is already here
             return 'present'
         else:
-            if not salt.utils.test_mode(test=test, **kwargs):
+            if not salt.utils.args.test_mode(test=test, **kwargs):
                 # The entry is new, add it to the end of the fstab
                 newline = (
                     '{0}\t{1}\t{2}\n'.format(
@@ -929,9 +966,9 @@ def set_automaster(
                )
                 lines.append(newline)
                 try:
-                    with salt.utils.fopen(config, 'w+') as ofile:
+                    with salt.utils.files.fopen(config, 'wb') as ofile:
                         # The line was changed, commit it!
-                        ofile.writelines(lines)
+                        ofile.writelines(salt.utils.data.encode(lines))
                 except (IOError, OSError):
                     raise CommandExecutionError(
                         'File not writable {0}'.format(
@@ -954,8 +991,9 @@ def automaster(config='/etc/auto_salt'):
     ret = {}
     if not os.path.isfile(config):
         return ret
-    with salt.utils.fopen(config) as ifile:
+    with salt.utils.files.fopen(config) as ifile:
         for line in ifile:
+            line = salt.utils.stringutils.to_unicode(line)
             if line.startswith('#'):
                 # Commented
                 continue
@@ -1114,12 +1152,12 @@ def is_fuse_exec(cmd):
 
         salt '*' mount.is_fuse_exec sshfs
     '''
-    cmd_path = _which(cmd)
+    cmd_path = salt.utils.path.which(cmd)
 
     # No point in running ldd on a command that doesn't exist
     if not cmd_path:
         return False
-    elif not _which('ldd'):
+    elif not salt.utils.path.which('ldd'):
         raise CommandNotFoundError('ldd')
 
     out = __salt__['cmd.run']('ldd {0}'.format(cmd_path), python_shell=False)
@@ -1149,8 +1187,9 @@ def swaps():
                              'used': (int(comps[3]) - int(comps[4])),
                              'priority': '-'}
     elif __grains__['os'] != 'OpenBSD':
-        with salt.utils.fopen('/proc/swaps') as fp_:
+        with salt.utils.files.fopen('/proc/swaps') as fp_:
             for line in fp_:
+                line = salt.utils.stringutils.to_unicode(line)
                 if line.startswith('Filename'):
                     continue
                 comps = line.split()
@@ -1260,3 +1299,91 @@ def is_mounted(name):
         return True
     else:
         return False
+
+
+def read_mount_cache(name):
+    '''
+    .. versionadded:: 2018.3.0
+
+    Provide information if the path is mounted
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' mount.read_mount_cache /mnt/share
+    '''
+    cache = salt.utils.mount.read_cache(__opts__)
+    if cache:
+        if 'mounts' in cache and cache['mounts']:
+            if name in cache['mounts']:
+                return cache['mounts'][name]
+    return {}
+
+
+def write_mount_cache(real_name,
+                      device,
+                      mkmnt,
+                      fstype,
+                      mount_opts):
+    '''
+    .. versionadded:: 2018.3.0
+
+    Provide information if the path is mounted
+
+    :param real_name:     The real name of the mount point where the device is mounted.
+    :param device:        The device that is being mounted.
+    :param mkmnt:         Whether or not the mount point should be created.
+    :param fstype:        The file system that is used.
+    :param mount_opts:    Additional options used when mounting the device.
+    :return:              Boolean if message was sent successfully.
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' mount.write_mount_cache /mnt/share /dev/sda1 False ext4 defaults,nosuid
+    '''
+    cache = salt.utils.mount.read_cache(__opts__)
+
+    if not cache:
+        cache = {}
+        cache['mounts'] = {}
+    else:
+        if 'mounts' not in cache:
+            cache['mounts'] = {}
+
+    cache['mounts'][real_name] = {'device': device,
+                                  'fstype': fstype,
+                                  'mkmnt': mkmnt,
+                                  'opts': mount_opts}
+
+    cache_write = salt.utils.mount.write_cache(cache, __opts__)
+    if cache_write:
+        return True
+    else:
+        raise CommandExecutionError('Unable to write mount cache.')
+
+
+def delete_mount_cache(real_name):
+    '''
+    .. versionadded:: 2018.3.0
+
+    Provide information if the path is mounted
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' mount.delete_mount_cache /mnt/share
+    '''
+    cache = salt.utils.mount.read_cache(__opts__)
+
+    if cache:
+        if 'mounts' in cache:
+            if real_name in cache['mounts']:
+                del cache['mounts'][real_name]
+                cache_write = salt.utils.mount.write_cache(cache, __opts__)
+                if not cache_write:
+                    raise CommandExecutionError('Unable to write mount cache.')
+    return True
