@@ -34,7 +34,7 @@ Module to provide MySQL compatibility to salt.
 '''
 
 # Import python libs
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 import time
 import logging
 import re
@@ -43,10 +43,12 @@ import shlex
 import os
 
 # Import salt libs
-import salt.utils
+import salt.utils.data
+import salt.utils.files
+import salt.utils.stringutils
 
 # Import third party libs
-import salt.ext.six as six
+from salt.ext import six
 # pylint: disable=import-error
 from salt.ext.six.moves import range, zip  # pylint: disable=no-name-in-module,redefined-builtin
 try:
@@ -370,7 +372,7 @@ def _grant_to_tokens(grant):
             - grant: [grant1, grant2] (ala SELECT, USAGE, etc)
             - database: MySQL DB
     '''
-    log.debug('_grant_to_tokens entry \'{0}\''.format(grant))
+    log.debug('_grant_to_tokens entry \'%s\'', grant)
     dict_mode = False
     if isinstance(grant, dict):
         dict_mode = True
@@ -414,7 +416,6 @@ def _grant_to_tokens(grant):
     position_tracker = 1  # Skip the initial 'GRANT' word token
     database = ''
     phrase = 'grants'
-    #log.debug('_grant_to_tokens lex analysis \'{0}\''.format(exploded_grant))
 
     for token in exploded_grant[position_tracker:]:
 
@@ -484,12 +485,11 @@ def _grant_to_tokens(grant):
             user = user.strip("'")
             host = host.strip("'")
         log.debug(
-            'grant to token \'{0}\'::\'{1}\'::\'{2}\'::\'{3}\''.format(
+            'grant to token \'%s\'::\'%s\'::\'%s\'::\'%s\'',
                 user,
                 host,
                 grant_tokens,
                 database
-            )
         )
     except UnboundLocalError:
         host = ''
@@ -541,10 +541,10 @@ def _execute(cur, qry, args=None):
     '''
     if args is None or args == {}:
         qry = qry.replace('%%', '%')
-        log.debug('Doing query: {0}'.format(qry))
+        log.debug('Doing query: %s', qry)
         return cur.execute(qry)
     else:
-        log.debug('Doing query: {0} args: {1} '.format(qry, repr(args)))
+        log.debug('Doing query: %s args: %s ', qry, repr(args))
         return cur.execute(qry, args)
 
 
@@ -644,7 +644,7 @@ def query(database, query, **connection_args):
         return {}
     cur = dbc.cursor()
     start = time.time()
-    log.debug('Using db: {0} to run query {1}'.format(database, query))
+    log.debug('Using db: %s to run query %s', database, query)
     try:
         affected = _execute(cur, query)
     except MySQLdb.OperationalError as exc:
@@ -687,11 +687,20 @@ def file_query(database, file_name, **connection_args):
 
     .. versionadded:: 2017.7.0
 
+    database
+
+        database to run script inside
+
+    file_name
+
+        File name of the script.  This can be on the minion, or a file that is reachable by the fileserver
+
     CLI Example:
 
     .. code-block:: bash
 
         salt '*' mysql.file_query mydb file_name=/tmp/sqlfile.sql
+        salt '*' mysql.file_query mydb file_name=salt://sqlfile.sql
 
     Return data:
 
@@ -700,15 +709,18 @@ def file_query(database, file_name, **connection_args):
         {'query time': {'human': '39.0ms', 'raw': '0.03899'}, 'rows affected': 1L}
 
     '''
+    if any(file_name.startswith(proto) for proto in ('salt://', 'http://', 'https://', 'swift://', 's3://')):
+        file_name = __salt__['cp.cache_file'](file_name)
+
     if os.path.exists(file_name):
-        with salt.utils.fopen(file_name, 'r') as ifile:
-            contents = ifile.read()
+        with salt.utils.files.fopen(file_name, 'r') as ifile:
+            contents = salt.utils.stringutils.to_unicode(ifile.read())
     else:
-        log.error('File "{0}" does not exist'.format(file_name))
+        log.error('File "%s" does not exist', file_name)
         return False
 
     query_string = ""
-    ret = {'rows returned': 0, 'columns': 0, 'results': 0, 'rows affected': 0, 'query time': {'raw': 0}}
+    ret = {'rows returned': 0, 'columns': [], 'results': [], 'rows affected': 0, 'query time': {'raw': 0}}
     for line in contents.splitlines():
         if re.match(r'--', line):  # ignore sql comments
             continue
@@ -728,16 +740,16 @@ def file_query(database, file_name, **connection_args):
             if 'rows returned' in query_result:
                 ret['rows returned'] += query_result['rows returned']
             if 'columns' in query_result:
-                ret['columns'] += query_result['columns']
+                ret['columns'].append(query_result['columns'])
             if 'results' in query_result:
-                ret['results'] += query_result['results']
+                ret['results'].append(query_result['results'])
             if 'rows affected' in query_result:
                 ret['rows affected'] += query_result['rows affected']
-    ret['query time']['human'] = str(round(float(ret['query time']['raw']), 2)) + 's'
+    ret['query time']['human'] = six.text_type(round(float(ret['query time']['raw']), 2)) + 's'
     ret['query time']['raw'] = round(float(ret['query time']['raw']), 5)
 
     # Remove empty keys in ret
-    ret = dict((k, v) for k, v in six.iteritems(ret) if v)
+    ret = {k: v for k, v in six.iteritems(ret) if v}
 
     return ret
 
@@ -986,7 +998,7 @@ def db_tables(name, **connection_args):
         salt '*' mysql.db_tables 'database'
     '''
     if not db_exists(name, **connection_args):
-        log.info('Database \'{0}\' does not exist'.format(name))
+        log.info('Database \'%s\' does not exist', name)
         return False
 
     dbc = _connect(**connection_args)
@@ -1064,7 +1076,7 @@ def db_create(name, character_set=None, collate=None, **connection_args):
     '''
     # check if db exists
     if db_exists(name, **connection_args):
-        log.info('DB \'{0}\' already exists'.format(name))
+        log.info('DB \'%s\' already exists', name)
         return False
 
     # db doesn't exist, proceed
@@ -1086,7 +1098,7 @@ def db_create(name, character_set=None, collate=None, **connection_args):
 
     try:
         if _execute(cur, qry, args):
-            log.info('DB \'{0}\' created'.format(name))
+            log.info('DB \'%s\' created', name)
             return True
     except MySQLdb.OperationalError as exc:
         err = 'MySQL Error {0}: {1}'.format(*exc)
@@ -1107,11 +1119,11 @@ def db_remove(name, **connection_args):
     '''
     # check if db exists
     if not db_exists(name, **connection_args):
-        log.info('DB \'{0}\' does not exist'.format(name))
+        log.info('DB \'%s\' does not exist', name)
         return False
 
     if name in ('mysql', 'information_scheme'):
-        log.info('DB \'{0}\' may not be removed'.format(name))
+        log.info('DB \'%s\' may not be removed', name)
         return False
 
     # db does exists, proceed
@@ -1131,10 +1143,10 @@ def db_remove(name, **connection_args):
         return False
 
     if not db_exists(name, **connection_args):
-        log.info('Database \'{0}\' has been removed'.format(name))
+        log.info('Database \'%s\' has been removed', name)
         return True
 
-    log.info('Database \'{0}\' has not been removed'.format(name))
+    log.info('Database \'%s\' has not been removed', name)
     return False
 
 
@@ -1215,15 +1227,15 @@ def user_exists(user,
     args['user'] = user
     args['host'] = host
 
-    if salt.utils.is_true(passwordless):
-        if salt.utils.is_true(unix_socket):
+    if salt.utils.data.is_true(passwordless):
+        if salt.utils.data.is_true(unix_socket):
             qry += ' AND plugin=%(unix_socket)s'
             args['unix_socket'] = 'unix_socket'
         else:
             qry += ' AND ' + password_column + ' = \'\''
     elif password:
         qry += ' AND ' + password_column + ' = PASSWORD(%(password)s)'
-        args['password'] = str(password)
+        args['password'] = six.text_type(password)
     elif password_hash:
         qry += ' AND ' + password_column + ' = %(password)s'
         args['password'] = password_hash
@@ -1322,7 +1334,7 @@ def user_create(user,
         salt '*' mysql.user_create 'username' 'hostname' allow_passwordless=True
     '''
     if user_exists(user, host, **connection_args):
-        log.info('User \'{0}\'@\'{1}\' already exists'.format(user, host))
+        log.info('User \'%s\'@\'%s\' already exists', user, host)
         return False
 
     dbc = _connect(**connection_args)
@@ -1339,12 +1351,12 @@ def user_create(user,
     args['host'] = host
     if password is not None:
         qry += ' IDENTIFIED BY %(password)s'
-        args['password'] = str(password)
+        args['password'] = six.text_type(password)
     elif password_hash is not None:
         qry += ' IDENTIFIED BY PASSWORD %(password)s'
         args['password'] = password_hash
-    elif salt.utils.is_true(allow_passwordless):
-        if salt.utils.is_true(unix_socket):
+    elif salt.utils.data.is_true(allow_passwordless):
+        if salt.utils.data.is_true(unix_socket):
             if host == 'localhost':
                 qry += ' IDENTIFIED VIA unix_socket'
             else:
@@ -1371,7 +1383,7 @@ def user_create(user,
         log.info(msg)
         return True
 
-    log.info('User \'{0}\'@\'{1}\' was not created'.format(user, host))
+    log.info('User \'%s\'@\'%s\' was not created', user, host)
     return False
 
 
@@ -1428,7 +1440,7 @@ def user_chpass(user,
     elif password_hash is not None:
         password_sql = '%(password)s'
         args['password'] = password_hash
-    elif not salt.utils.is_true(allow_passwordless):
+    elif not salt.utils.data.is_true(allow_passwordless):
         log.error('password or password_hash must be specified, unless '
                   'allow_passwordless=True')
         return False
@@ -1448,8 +1460,8 @@ def user_chpass(user,
            ' WHERE User=%(user)s AND Host = %(host)s;')
     args['user'] = user
     args['host'] = host
-    if salt.utils.is_true(allow_passwordless) and \
-            salt.utils.is_true(unix_socket):
+    if salt.utils.data.is_true(allow_passwordless) and \
+            salt.utils.data.is_true(unix_socket):
         if host == 'localhost':
             qry = ('UPDATE mysql.user SET ' + password_column + '='
                    + password_sql + ', plugin=%(unix_socket)s' +
@@ -1468,18 +1480,16 @@ def user_chpass(user,
     if result:
         _execute(cur, 'FLUSH PRIVILEGES;')
         log.info(
-            'Password for user \'{0}\'@\'{1}\' has been {2}'.format(
+            'Password for user \'%s\'@\'%s\' has been %s',
                 user, host,
                 'changed' if any((password, password_hash)) else 'cleared'
-            )
         )
         return True
 
     log.info(
-        'Password for user \'{0}\'@\'{1}\' was not {2}'.format(
+        'Password for user \'%s\'@\'%s\' was not %s',
             user, host,
             'changed' if any((password, password_hash)) else 'cleared'
-        )
     )
     return False
 
@@ -1514,10 +1524,10 @@ def user_remove(user,
         return False
 
     if not user_exists(user, host, **connection_args):
-        log.info('User \'{0}\'@\'{1}\' has been removed'.format(user, host))
+        log.info('User \'%s\'@\'%s\' has been removed', user, host)
         return True
 
-    log.info('User \'{0}\'@\'{1}\' has NOT been removed'.format(user, host))
+    log.info('User \'%s\'@\'%s\' has NOT been removed', user, host)
     return False
 
 
@@ -1556,12 +1566,10 @@ def db_check(name,
         # we need to check all tables
         tables = db_tables(name, **connection_args)
         for table in tables:
-            log.info(
-                'Checking table \'{0}\' in db \'{1}\'..'.format(name, table)
-            )
+            log.info('Checking table \'%s\' in db \'%s\'..', name, table)
             ret.append(__check_table(name, table, **connection_args))
     else:
-        log.info('Checking table \'{0}\' in db \'{1}\'..'.format(name, table))
+        log.info('Checking table \'%s\' in db \'%s\'..', name, table)
         ret = __check_table(name, table, **connection_args)
     return ret
 
@@ -1583,12 +1591,10 @@ def db_repair(name,
         # we need to repair all tables
         tables = db_tables(name, **connection_args)
         for table in tables:
-            log.info(
-                'Repairing table \'{0}\' in db \'{1}\'..'.format(name, table)
-            )
+            log.info('Repairing table \'%s\' in db \'%s\'..', name, table)
             ret.append(__repair_table(name, table, **connection_args))
     else:
-        log.info('Repairing table \'{0}\' in db \'{1}\'..'.format(name, table))
+        log.info('Repairing table \'%s\' in db \'%s\'..', name, table)
         ret = __repair_table(name, table, **connection_args)
     return ret
 
@@ -1610,14 +1616,10 @@ def db_optimize(name,
         # we need to optimize all tables
         tables = db_tables(name, **connection_args)
         for table in tables:
-            log.info(
-                'Optimizing table \'{0}\' in db \'{1}\'..'.format(name, table)
-            )
+            log.info('Optimizing table \'%s\' in db \'%s\'..', name, table)
             ret.append(__optimize_table(name, table, **connection_args))
     else:
-        log.info(
-            'Optimizing table \'{0}\' in db \'{1}\'..'.format(name, table)
-        )
+        log.info('Optimizing table \'%s\' in db \'%s\'..', name, table)
         ret = __optimize_table(name, table, **connection_args)
     return ret
 
@@ -1689,11 +1691,11 @@ def __grant_generate(grant,
     table = db_part[2]
 
     if escape:
-        if dbc is not '*':
+        if dbc != '*':
             # _ and % are authorized on GRANT queries and should get escaped
             # on the db name, but only if not requesting a table level grant
-            dbc = quote_identifier(dbc, for_grants=(table is '*'))
-        if table is not '*':
+            dbc = quote_identifier(dbc, for_grants=(table == '*'))
+        if table != '*':
             table = quote_identifier(table)
     # identifiers cannot be used as values, and same thing for grants
     qry = 'GRANT {0} ON {1}.{2} TO %(user)s@%(host)s'.format(grant, dbc, table)
@@ -1702,9 +1704,9 @@ def __grant_generate(grant,
     args['host'] = host
     if isinstance(ssl_option, list) and len(ssl_option):
         qry += __ssl_option_sanitize(ssl_option)
-    if salt.utils.is_true(grant_option):
+    if salt.utils.data.is_true(grant_option):
         qry += ' WITH GRANT OPTION'
-    log.debug('Grant Query generated: {0} args {1}'.format(qry, repr(args)))
+    log.debug('Grant Query generated: %s args %s', qry, repr(args))
     return {'qry': qry, 'args': args}
 
 
@@ -1720,7 +1722,7 @@ def user_grants(user,
         salt '*' mysql.user_grants 'frank' 'localhost'
     '''
     if not user_exists(user, host, **connection_args):
-        log.info('User \'{0}\'@\'{1}\' does not exist'.format(user, host))
+        log.info('User \'%s\'@\'%s\' does not exist', user, host)
         return False
 
     dbc = _connect(**connection_args)
@@ -1788,16 +1790,15 @@ def grant_exists(grant,
             if not target_tokens:  # Avoid the overhead of re-calc in loop
                 target_tokens = _grant_to_tokens(target)
             grant_tokens = _grant_to_tokens(grant)
+            grant_tokens_database = grant_tokens['database'].replace('"', '').replace('\\', '').replace('`', '')
+            target_tokens_database = target_tokens['database'].replace('"', '').replace('\\', '').replace('`', '')
             if grant_tokens['user'] == target_tokens['user'] and \
-                    grant_tokens['database'] == target_tokens['database'] and \
+                    grant_tokens_database == target_tokens_database and \
                     grant_tokens['host'] == target_tokens['host'] and \
                     set(grant_tokens['grant']) >= set(target_tokens['grant']):
                 return True
             else:
-                log.debug('grants mismatch \'{0}\'<>\'{1}\''.format(
-                    grant_tokens,
-                    target_tokens
-                ))
+                log.debug('grants mismatch \'%s\'<>\'%s\'', grant_tokens, target_tokens)
 
         except Exception as exc:  # Fallback to strict parsing
             log.exception(exc)
@@ -1852,16 +1853,14 @@ def grant_add(grant,
             grant, database, user, host, grant_option, escape,
             **connection_args):
         log.info(
-            'Grant \'{0}\' on \'{1}\' for user \'{2}\' has been added'.format(
-                grant, database, user
-            )
+            'Grant \'%s\' on \'%s\' for user \'%s\' has been added',
+            grant, database, user
         )
         return True
 
     log.info(
-        'Grant \'{0}\' on \'{1}\' for user \'{2}\' has NOT been added'.format(
-            grant, database, user
-        )
+        'Grant \'%s\' on \'%s\' for user \'%s\' has NOT been added',
+        grant, database, user
     )
     return False
 
@@ -1890,22 +1889,22 @@ def grant_revoke(grant,
 
     grant = __grant_normalize(grant)
 
-    if salt.utils.is_true(grant_option):
+    if salt.utils.data.is_true(grant_option):
         grant += ', GRANT OPTION'
 
     db_part = database.rpartition('.')
     dbc = db_part[0]
     table = db_part[2]
-    if dbc is not '*':
+    if dbc != '*':
         # _ and % are authorized on GRANT queries and should get escaped
         # on the db name, but only if not requesting a table level grant
-        s_database = quote_identifier(dbc, for_grants=(table is '*'))
-    if dbc is '*':
+        s_database = quote_identifier(dbc, for_grants=(table == '*'))
+    if dbc == '*':
         # add revoke for *.*
         # before the modification query send to mysql will looks like
         # REVOKE SELECT ON `*`.* FROM %(user)s@%(host)s
         s_database = dbc
-    if table is not '*':
+    if table != '*':
         table = quote_identifier(table)
     # identifiers cannot be used as values, same thing for grants
     qry = 'REVOKE {0} ON {1}.{2} FROM %(user)s@%(host)s;'.format(
@@ -1933,15 +1932,13 @@ def grant_revoke(grant,
                         escape,
                         **connection_args):
         log.info(
-            'Grant \'{0}\' on \'{1}\' for user \'{2}\' has been '
-            'revoked'.format(grant, database, user)
-        )
+            'Grant \'%s\' on \'%s\' for user \'%s\' has been '
+            'revoked', grant, database, user)
         return True
 
     log.info(
-        'Grant \'{0}\' on \'{1}\' for user \'{2}\' has NOT been '
-        'revoked'.format(grant, database, user)
-    )
+        'Grant \'%s\' on \'%s\' for user \'%s\' has NOT been '
+        'revoked', grant, database, user)
     return False
 
 
@@ -1997,24 +1994,24 @@ def __do_query_into_hash(conn, sql_str):
 
     '''
     mod = sys._getframe().f_code.co_name
-    log.debug('{0}<--({1})'.format(mod, sql_str))
+    log.debug('%s<--(%s)', mod, sql_str)
 
     rtn_results = []
 
     try:
         cursor = conn.cursor()
     except MySQLdb.MySQLError:
-        log.error('{0}: Can\'t get cursor for SQL->{1}'.format(mod, sql_str))
+        log.error('%s: Can\'t get cursor for SQL->%s', mod, sql_str)
         cursor.close()
-        log.debug('{0}-->'.format(mod))
+        log.debug('%s-->', mod)
         return rtn_results
 
     try:
         _execute(cursor, sql_str)
     except MySQLdb.MySQLError:
-        log.error('{0}: try to execute : SQL->{1}'.format(mod, sql_str))
+        log.error('%s: try to execute : SQL->%s', mod, sql_str)
         cursor.close()
-        log.debug('{0}-->'.format(mod))
+        log.debug('%s-->', mod)
         return rtn_results
 
     qrs = cursor.fetchall()
@@ -2030,7 +2027,7 @@ def __do_query_into_hash(conn, sql_str):
         rtn_results.append(row)
 
     cursor.close()
-    log.debug('{0}-->'.format(mod))
+    log.debug('%s-->', mod)
     return rtn_results
 
 
@@ -2053,7 +2050,7 @@ def get_master_status(**connection_args):
 
     '''
     mod = sys._getframe().f_code.co_name
-    log.debug('{0}<--'.format(mod))
+    log.debug('%s<--', mod)
     conn = _connect(**connection_args)
     if conn is None:
         return []
@@ -2064,7 +2061,7 @@ def get_master_status(**connection_args):
     if len(rtnv) == 0:
         rtnv.append([])
 
-    log.debug('{0}-->{1}'.format(mod, len(rtnv[0])))
+    log.debug('%s-->%s', mod, len(rtnv[0]))
     return rtnv[0]
 
 
@@ -2123,7 +2120,7 @@ def get_slave_status(**connection_args):
 
     '''
     mod = sys._getframe().f_code.co_name
-    log.debug('{0}<--'.format(mod))
+    log.debug('%s<--', mod)
     conn = _connect(**connection_args)
     if conn is None:
         return []
@@ -2134,7 +2131,7 @@ def get_slave_status(**connection_args):
     if len(rtnv) == 0:
         rtnv.append([])
 
-    log.debug('{0}-->{1}'.format(mod, len(rtnv[0])))
+    log.debug('%s-->%s', mod, len(rtnv[0]))
     return rtnv[0]
 
 
@@ -2153,7 +2150,7 @@ def showvariables(**connection_args):
 
     '''
     mod = sys._getframe().f_code.co_name
-    log.debug('{0}<--'.format(mod))
+    log.debug('%s<--', mod)
     conn = _connect(**connection_args)
     if conn is None:
         return []
@@ -2162,7 +2159,7 @@ def showvariables(**connection_args):
     if len(rtnv) == 0:
         rtnv.append([])
 
-    log.debug('{0}-->{1}'.format(mod, len(rtnv[0])))
+    log.debug('%s-->%s', mod, len(rtnv[0]))
     return rtnv
 
 
@@ -2181,7 +2178,7 @@ def showglobal(**connection_args):
 
     '''
     mod = sys._getframe().f_code.co_name
-    log.debug('{0}<--'.format(mod))
+    log.debug('%s<--', mod)
     conn = _connect(**connection_args)
     if conn is None:
         return []
@@ -2190,5 +2187,5 @@ def showglobal(**connection_args):
     if len(rtnv) == 0:
         rtnv.append([])
 
-    log.debug('{0}-->{1}'.format(mod, len(rtnv[0])))
+    log.debug('%s-->%s', mod, len(rtnv[0]))
     return rtnv
