@@ -649,21 +649,6 @@ def extracted(name,
     # Remove pub kwargs as they're irrelevant here.
     kwargs = salt.utils.args.clean_kwargs(**kwargs)
 
-    if 'keep_source' in kwargs and 'keep' in kwargs:
-        ret.setdefault('warnings', []).append(
-            'Both \'keep_source\' and \'keep\' were used. Since these both '
-            'do the same thing, \'keep\' was ignored.'
-        )
-        keep_source = bool(kwargs.pop('keep_source'))
-        kwargs.pop('keep')
-    elif 'keep_source' in kwargs:
-        keep_source = bool(kwargs.pop('keep_source'))
-    elif 'keep' in kwargs:
-        keep_source = bool(kwargs.pop('keep'))
-    else:
-        # Neither was passed, default is True
-        keep_source = True
-
     if 'keep_source' in kwargs and 'keep' in kwargs:
         ret.setdefault('warnings', []).append(
             'Both \'keep_source\' and \'keep\' were used. Since these both '
@@ -696,7 +681,7 @@ def extracted(name,
         # True
         # >>> os.path.isfile('/tmp/foo.txt/')
         # False
-        name = name.rstrip('/')
+        name = name.rstrip(os.sep)
         if os.path.isfile(name):
             ret['comment'] = '{0} exists and is not a directory'.format(name)
             return ret
@@ -729,6 +714,11 @@ def extracted(name,
                 )
                 return ret
 
+    if if_missing is not None and os.path.exists(if_missing):
+        ret['result'] = True
+        ret['comment'] = 'Path {0} exists'.format(if_missing)
+        return ret
+
     if user or group:
         if salt.utils.platform.is_windows():
             ret['comment'] = \
@@ -1511,7 +1501,7 @@ def extracted(name,
         if not if_missing:
             # If is_missing was used, and both a) the archive had never been
             # extracted, and b) the path referred to by if_missing exists, then
-            # enforce_missing would contain paths of top_levle dirs/files that
+            # enforce_missing would contain paths of top_level dirs/files that
             # _would_ have been extracted. Since if_missing can be used as a
             # semaphore to conditionally extract, we don't want to make this a
             # case where the state fails, so we only fail the state if
