@@ -4,7 +4,7 @@ Managing Images in OpenStack Glance
 ===================================
 '''
 # Import python libs
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 import logging
 import time
 
