@@ -37,18 +37,18 @@ For the sake of brevity, examples for each state assume that jboss_config is con
 '''
 
 # Import python libs
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 import time
 import logging
 import re
 import traceback
 
 # Import Salt libs
-from salt.utils import dictdiffer
+import salt.utils.dictdiffer as dictdiffer
 from salt.exceptions import CommandExecutionError
 
 # Import 3rd-party libs
-import salt.ext.six as six
+from salt.ext import six
 
 log = logging.getLogger(__name__)
 
@@ -151,8 +151,8 @@ def datasource_exists(name, jboss_config, datasource_properties, recreate=False,
             raise CommandExecutionError('Unable to handle error: {0}'.format(ds_result['failure-description']))
 
     if ret['result']:
-        log.debug("ds_new_properties=%s", str(ds_new_properties))
-        log.debug("ds_current_properties=%s", str(ds_current_properties))
+        log.debug("ds_new_properties=%s", ds_new_properties)
+        log.debug("ds_current_properties=%s", ds_current_properties)
         diff = dictdiffer.diff(ds_new_properties, ds_current_properties)
 
         added = diff.added()
@@ -177,10 +177,10 @@ def datasource_exists(name, jboss_config, datasource_properties, recreate=False,
 
 
 def __format_ds_changes(keys, old_dict, new_dict):
-    log.debug("__format_ds_changes(keys=%s, old_dict=%s, new_dict=%s)", str(keys), str(old_dict), str(new_dict))
+    log.debug("__format_ds_changes(keys=%s, old_dict=%s, new_dict=%s)", keys, old_dict, new_dict)
     changes = ''
     for key in keys:
-        log.debug("key=%s", str(key))
+        log.debug("key=%s", key)
         if key in old_dict and key in new_dict:
             changes += key+':'+__get_ds_value(old_dict, key)+'->'+__get_ds_value(new_dict, key)+'\n'
         elif key in old_dict:
@@ -197,7 +197,7 @@ def __get_ds_value(dct, key):
     elif dct[key] is None:
         return 'undefined'
     else:
-        return str(dct[key])
+        return six.text_type(dct[key])
 
 
 def bindings_exist(name, jboss_config, bindings, profile=None):
@@ -226,7 +226,7 @@ def bindings_exist(name, jboss_config, bindings, profile=None):
 
     '''
     log.debug(" ======================== STATE: jboss7.bindings_exist (name: %s) (profile: %s) ", name, profile)
-    log.debug('bindings='+str(bindings))
+    log.debug('bindings=%s', bindings)
     ret = {'name': name,
            'result': True,
            'changes': {},
@@ -234,7 +234,7 @@ def bindings_exist(name, jboss_config, bindings, profile=None):
 
     has_changed = False
     for key in bindings:
-        value = str(bindings[key])
+        value = six.text_type(bindings[key])
         query_result = __salt__['jboss7.read_simple_binding'](binding_name=key, jboss_config=jboss_config, profile=profile)
         if query_result['success']:
             current_value = query_result['result']['value']
@@ -353,7 +353,7 @@ def deployed(name, jboss_config, salt_source=None):
         ret['changes']['undeployed'] = deployment
 
     deploy_result = __salt__['jboss7.deploy'](jboss_config=jboss_config, source_file=resolved_source)
-    log.debug('deploy_result=%s', str(deploy_result))
+    log.debug('deploy_result=%s', deploy_result)
     if deploy_result['success']:
         comment = __append_comment(new_comment='Deployment completed.', current_comment=comment)
         ret['comment'] = comment
@@ -448,9 +448,9 @@ def __get_artifact(salt_source):
                 log.debug(traceback.format_exc())
                 comment = 'Unable to manage file: {0}'.format(e)
 
-            else:
-                resolved_source = salt_source['target_file']
-                comment = ''
+        else:
+            resolved_source = salt_source['target_file']
+            comment = ''
 
     return resolved_source, comment
 
