@@ -15,27 +15,39 @@ Utils for the NAPALM modules and proxy.
 .. versionadded:: 2017.7.0
 '''
 
-from __future__ import absolute_import
-
+# Import Python libs
+from __future__ import absolute_import, unicode_literals, print_function
 import traceback
 import logging
 import importlib
 from functools import wraps
-log = logging.getLogger(__file__)
 
-import salt.utils
+# Import Salt libs
+from salt.ext import six as six
 import salt.output
+import salt.utils.platform
 
-# Import third party lib
+# Import third party libs
 try:
     # will try to import NAPALM
     # https://github.com/napalm-automation/napalm
     # pylint: disable=W0611
-    import napalm_base
+    import napalm
+    import napalm.base as napalm_base
     # pylint: enable=W0611
     HAS_NAPALM = True
+    HAS_NAPALM_BASE = False  # doesn't matter anymore, but needed for the logic below
+    try:
+        NAPALM_MAJOR = int(napalm.__version__.split('.')[0])
+    except AttributeError:
+        NAPALM_MAJOR = 0
 except ImportError:
     HAS_NAPALM = False
+    try:
+        import napalm_base
+        HAS_NAPALM_BASE = True
+    except ImportError:
+        HAS_NAPALM_BASE = False
 
 try:
     # try importing ConnectionClosedException
@@ -46,14 +58,14 @@ try:
 except ImportError:
     HAS_CONN_CLOSED_EXC_CLASS = False
 
-from salt.ext import six as six
+log = logging.getLogger(__file__)
 
 
 def is_proxy(opts):
     '''
     Is this a NAPALM proxy?
     '''
-    return salt.utils.is_proxy() and opts.get('proxy', {}).get('proxytype') == 'napalm'
+    return salt.utils.platform.is_proxy() and opts.get('proxy', {}).get('proxytype') == 'napalm'
 
 
 def is_always_alive(opts):
@@ -74,14 +86,14 @@ def is_minion(opts):
     '''
     Is this a NAPALM straight minion?
     '''
-    return not salt.utils.is_proxy() and 'napalm' in opts
+    return not salt.utils.platform.is_proxy() and 'napalm' in opts
 
 
 def virtual(opts, virtualname, filename):
     '''
     Returns the __virtual__.
     '''
-    if HAS_NAPALM and (is_proxy(opts) or is_minion(opts)):
+    if ((HAS_NAPALM and NAPALM_MAJOR >= 2) or HAS_NAPALM_BASE) and (is_proxy(opts) or is_minion(opts)):
         return virtualname
     else:
         return (
@@ -176,14 +188,14 @@ def call(napalm_device, method, *args, **kwargs):
             comment = 'Disconnected from {device}. Trying to reconnect.'.format(device=hostname)
             log.error(err_tb)
             log.error(comment)
-            log.debug('Clearing the connection with {device}'.format(device=hostname))
+            log.debug('Clearing the connection with %s', hostname)
             call(napalm_device, 'close', __retry=False)  # safely close the connection
             # Make sure we don't leave any TCP connection open behind
             #   if we fail to close properly, we might not be able to access the
-            log.debug('Re-opening the connection with {device}'.format(device=hostname))
+            log.debug('Re-opening the connection with %s', hostname)
             call(napalm_device, 'open', __retry=False)
-            log.debug('Connection re-opened with {device}'.format(device=hostname))
-            log.debug('Re-executing {method}'.format(method=method))
+            log.debug('Connection re-opened with %s', hostname)
+            log.debug('Re-executing %s', method)
             return call(napalm_device, method, *args, **kwargs)
             # If still not able to reconnect and execute the task,
             #   the proxy keepalive feature (if enabled) will attempt
@@ -226,7 +238,7 @@ def call(napalm_device, method, *args, **kwargs):
             # either running in a not-always-alive proxy
             # either running in a regular minion
             # close the connection when the call is over
-            # unless the CLOSE is explicitely set as False
+            # unless the CLOSE is explicitly set as False
             napalm_device['DRIVER'].close()
     return {
         'out': out,
@@ -247,7 +259,7 @@ def get_device_opts(opts, salt_obj=None):
     if opts.get('proxy') or opts.get('napalm'):
         opts['multiprocessing'] = device_dict.get('multiprocessing', False)
         # Most NAPALM drivers are SSH-based, so multiprocessing should default to False.
-        # But the user can be allows to have a different value for the multiprocessing, which will
+        # But the user can be allows one to have a different value for the multiprocessing, which will
         #   override the opts.
     if salt_obj and not device_dict:
         # get the connection details from the opts
@@ -304,7 +316,9 @@ def get_device(opts, salt_obj=None):
         try:
             provider_lib = importlib.import_module(network_device.get('PROVIDER'))
         except ImportError as ierr:
-            log.error('Unable to import {0}'.format(network_device.get('PROVIDER')), exc_info=True)
+            log.error('Unable to import %s',
+                      network_device.get('PROVIDER'),
+                      exc_info=True)
             log.error('Falling back to napalm-base')
     _driver_ = provider_lib.get_network_driver(network_device.get('DRIVER_NAME'))
     try:
@@ -327,9 +341,7 @@ def get_device(opts, salt_obj=None):
         )
         log.error(base_err_msg)
         log.error(
-            "Please check error: {error}".format(
-                error=error
-            )
+            "Please check error: %s", error
         )
         raise napalm_base.exceptions.ConnectionException(base_err_msg)
     return network_device
@@ -355,11 +367,11 @@ def proxy_napalm_wrap(func):
         # the execution modules will make use of this variable from now on
         # previously they were accessing the device properties through the __proxy__ object
         always_alive = opts.get('proxy', {}).get('always_alive', True)
-        if salt.utils.is_proxy() and always_alive:
+        if salt.utils.platform.is_proxy() and always_alive:
             # if it is running in a proxy and it's using the default always alive behaviour,
             # will get the cached copy of the network device
             wrapped_global_namespace['napalm_device'] = proxy['napalm.get_device']()
-        elif salt.utils.is_proxy() and not always_alive:
+        elif salt.utils.platform.is_proxy() and not always_alive:
             # if still proxy, but the user does not want the SSH session always alive
             # get a new device instance
             # which establishes a new connection
@@ -375,12 +387,12 @@ def proxy_napalm_wrap(func):
                 except napalm_base.exceptions.ConnectionException as nce:
                     log.error(nce)
                     return '{base_msg}. See log for details.'.format(
-                        base_msg=str(nce.msg)
+                        base_msg=six.text_type(nce.msg)
                     )
             else:
                 # in case the `inherit_napalm_device` is set
                 # and it also has a non-empty value,
-                # the global var `napalm_device` will be overriden.
+                # the global var `napalm_device` will be overridden.
                 # this is extremely important for configuration-related features
                 # as all actions must be issued within the same configuration session
                 # otherwise we risk to open multiple sessions
@@ -401,12 +413,12 @@ def proxy_napalm_wrap(func):
                 except napalm_base.exceptions.ConnectionException as nce:
                     log.error(nce)
                     return '{base_msg}. See log for details.'.format(
-                        base_msg=str(nce.msg)
+                        base_msg=six.text_type(nce.msg)
                     )
             else:
                 # in case the `inherit_napalm_device` is set
                 # and it also has a non-empty value,
-                # the global var `napalm_device` will be overriden.
+                # the global var `napalm_device` will be overridden.
                 # this is extremely important for configuration-related features
                 # as all actions must be issued within the same configuration session
                 # otherwise we risk to open multiple sessions
